@@ -1,8 +1,8 @@
 #' 2pl LSIRM model fixing gamma to 1.
-#' 
-#' @description \link{lsirm2pl_fixed_gamma} is used to fit 2pl LSIRM model fixing gamma to 1. 
-#' \link{lsirm2pl_fixed_gamma} factorizes item response matrix into column-wise item effect, row-wise respondent effect and further embeds interaction effect in a latent space. Unlike 1pl model, 2pl model assumes the item effect can vary according to respondent, allowing additional parameter multiplied with respondent effect. The resulting latent space provides an interaction map that represents interactions between respondents and items. 
-#' 
+#'
+#' @description \link{lsirm2pl_fixed_gamma} is used to fit 2pl LSIRM model fixing gamma to 1.
+#' \link{lsirm2pl_fixed_gamma} factorizes item response matrix into column-wise item effect, row-wise respondent effect and further embeds interaction effect in a latent space. Unlike 1pl model, 2pl model assumes the item effect can vary according to respondent, allowing additional parameter multiplied with respondent effect. The resulting latent space provides an interaction map that represents interactions between respondents and items.
+#'
 #' @param data Matrix; binary item response matrix to be analyzed. Each row is assumed to be respondent and its column values are assumed to be response to the corresponding item.
 #' @param ndim Numeric; dimension of latent space. default value is 2.
 #' @param niter Numeric; number of iterations to run MCMC sampling. default value is 15000.
@@ -21,9 +21,13 @@
 #' @param pr_sd_alpha Numeric; mean of normal prior for beta. default value is 1.0.
 #' @param pr_a_theta Numeric; shape parameter of inverse gamma prior for variance of theta. default value is 0.001.
 #' @param pr_b_theta Numeric; scale parameter of inverse gamma prior for variance of theta. default value is 0.001.
-#' 
-#' 
+#'
+#'
 #' @return \code{lsirm2pl_fixed_gamma} returns an object of  list containing the following components:
+#'  \item{data}{data frame or matrix containing the variables in the model.}
+#'  \item{bic}{Numeric value with the corresponding BIC.}
+#'  \item{mcmc_inf}{number of mcmc iteration, burn-in periods, and thinning intervals.}
+#'  \item{map_inf}{value of log maximum a posterior and iteration number which have log maximum a posterior.}
 #'  \item{beta_estimate}{posterior estimation of beta.}
 #'  \item{theta_estimate}{posterior estimation of theta.}
 #'  \item{sigma_theta_estimate}{posterior estimation of standard deviation of theta.}
@@ -41,35 +45,42 @@
 #'  \item{accept_w}{accept ratio of w.}
 #'  \item{accept_z}{accept ratio of z.}
 #'  \item{accept_alpha}{accept ratio of alpha.}
-#' 
+#'
 #' @details \code{lsirm2pl_fixed_gamma} models the probability of correct response by respondent \eqn{j} to item \eqn{i} with item effect \eqn{\beta_i}, respondent effect \eqn{\theta_j} and the distance between latent position \eqn{w_i} of item \eqn{i} and latent position \eqn{z_j} of respondent \eqn{j} in the shared metric space. For 2pl model, the the item effect is assumed to have additional discrimination parameter \eqn{\alpha_i} multiplied by \eqn{\theta_j}: \deqn{logit(P(Y_{j,i} = 1|\theta_j,\alpha_i,\beta_i,z_j,w_i))=\theta_j*\alpha_i+\beta_i-||z_j-w_i||}
-#' 
-#' @examples 
+#'
+#' @examples
 #' \donttest{
 #' # generate example item response matrix
 #' data     <- matrix(rbinom(500, size = 1, prob = 0.5),ncol=10,nrow=50)
-#' 
+#'
 #' lsirm_result <- lsirm2pl_fixed_gamma(data)
-#' 
+#'
 #' # The code following can achieve the same result.
-#' lsirm_result <- lsirm2pl(data, spikenslab = FALSE, fixed_gamma = TRUE)
+#' lsirm_result <- lsirm(data ~ lsirm2pl(spikenslab = FALSE, fixed_gamma = TRUE))
 #' }
-#' 
 #' @export
 lsirm2pl_fixed_gamma = function(data, ndim = 2, niter = 15000, nburn = 2500, nthin = 5, nprint = 500,
-                               jump_beta = 0.4, jump_theta = 1, jump_alpha = 1.0, jump_z = 0.5, jump_w = 0.5,
-                               pr_mean_beta = 0, pr_sd_beta = 1, pr_mean_theta = 0, 
-                               pr_mean_alpha = 0.5, pr_sd_alpha = 1, pr_a_theta = 0.001, pr_b_theta = 0.001){
-  
-  output <- lsirm2pl_fixed_gamma_cpp(data, ndim, niter, nburn, nthin, nprint,
-                                    jump_beta, jump_theta, jump_alpha, jump_z, jump_w,
-                                    pr_mean_beta, pr_sd_beta, pr_mean_theta, 
-                                    pr_mean_alpha, pr_sd_alpha, pr_a_theta, pr_b_theta)
+                                jump_beta = 0.4, jump_theta = 1, jump_alpha = 1.0, jump_z = 0.5, jump_w = 0.5,
+                                pr_mean_beta = 0, pr_sd_beta = 1, pr_mean_theta = 0,
+                                pr_mean_alpha = 0.5, pr_sd_alpha = 1, pr_a_theta = 0.001, pr_b_theta = 0.001){
+
+  if(is.data.frame(data)){
+    cname = colnames(data)
+  }else{
+    cname = paste("item", 1:ncol(data), sep=" ")
+  }
+
+  output <- lsirm2pl_fixed_gamma_cpp(as.matrix(data), ndim, niter, nburn, nthin, nprint,
+                                     jump_beta, jump_theta, jump_alpha, jump_z, jump_w,
+                                     pr_mean_beta, pr_sd_beta, pr_mean_theta,
+                                     pr_mean_alpha, pr_sd_alpha, pr_a_theta, pr_b_theta)
+  mcmc.inf = list(nburn=nburn, niter=niter, nthin=nthin)
   nsample <- nrow(data)
   nitem <- ncol(data)
-  
+
   nmcmc = as.integer((niter - nburn) / nthin)
   max.address = min(which.max(output$map))
+  map.inf = data.frame(value = output$map[which.max(output$map)], iter = which.max(output$map))
   w.star = output$w[max.address,,]
   z.star = output$z[max.address,,]
   w.proc = array(0,dim=c(nmcmc,nitem,ndim))
@@ -79,35 +90,52 @@ lsirm2pl_fixed_gamma = function(data, ndim = 2, niter = 15000, nburn = 2500, nth
     z.iter = output$z[iter,,]
     if(iter != max.address) z.proc[iter,,] = procrustes(z.iter,z.star)$X.new
     else z.proc[iter,,] = z.iter
-    
+
     w.iter = output$w[iter,,]
     if(iter != max.address) w.proc[iter,,] = procrustes(w.iter,w.star)$X.new
     else w.proc[iter,,] = w.iter
   }
-  
+
   w.est = colMeans(w.proc, dims = 1)
   z.est = colMeans(z.proc, dims = 1)
-  
+
   beta.estimate = apply(output$beta, 2, mean)
   theta.estimate = apply(output$theta, 2, mean)
   alpha.estimate = apply(output$alpha, 2, mean)
   sigma_theta.estimate = mean(output$sigma_theta)
-  
-  return(list(beta_estimate  = beta.estimate,
-              theta_estimate = theta.estimate,
-              sigma_theta_estimate    = sigma_theta.estimate,
-              alhpa_estimate = alpha.estimate,
-              z_estimate     = z.est,
-              w_estimate     = w.est,
-              beta           = output$beta,
-              theta          = output$theta,
-              theta_sd       = output$sigma_theta,
-              alpha          = output$alpha,
-              z              = z.proc,
-              w              = w.proc,
-              accept_beta    = output$accept_beta,
-              accept_theta   = output$accept_theta,
-              accept_alpha   = output$accept_alpha,
-              accept_w       = output$accept_w,
-              accept_z       = output$accept_z))
-}
\ No newline at end of file
+
+  beta.summary = data.frame(cbind(apply(output$beta, 2, mean), t(apply(output$beta, 2, function(x) quantile(x, probs = c(0.025, 0.975))))))
+  colnames(beta.summary) <- c("Estimate", "2.5%", "97.5%")
+  rownames(beta.summary) <- cname
+
+  # Calculate BIC
+  log_like = log_likelihood_2pl_cpp(as.matrix(data), ndim, as.matrix(beta.estimate), as.matrix(alpha.estimate), as.matrix(theta.estimate), 1, z.est, w.est, 99)
+  p = 2 * nitem + nsample + 1 + ndim * nitem + ndim * nsample
+  bic = -2 * log_like[[1]] + p * log(nsample * nsample)
+
+  result <- list(data = data,
+              bic = bic,
+                 mcmc_inf = mcmc.inf,
+                 map_inf = map.inf,
+                 beta_estimate  = beta.estimate,
+                 beta_summary = beta.summary,
+                 theta_estimate = theta.estimate,
+                 sigma_theta_estimate    = sigma_theta.estimate,
+                 alhpa_estimate = alpha.estimate,
+                 z_estimate     = z.est,
+                 w_estimate     = w.est,
+                 beta           = output$beta,
+                 theta          = output$theta,
+                 theta_sd       = output$sigma_theta,
+                 alpha          = output$alpha,
+                 z              = z.proc,
+                 w              = w.proc,
+                 accept_beta    = output$accept_beta,
+                 accept_theta   = output$accept_theta,
+                 accept_alpha   = output$accept_alpha,
+                 accept_w       = output$accept_w,
+                 accept_z       = output$accept_z)
+  class(result) = "lsirm"
+
+  return(result)
+}
